-------------------------------------------------------
-- Design Name      : mb_hv_ctrl_top 
-- File Name        : mb_hv_ctrl_top.vhd
-- Device           : CPLD, XC2C128-7VQG100I
-- Function         : mDOM mainboard, PMT bases, control
--                    
-- Coder            : K.-H. Sulanke, DESY, 2020-12-21
-------------------------------------------------------
-- Rev. 1


library IEEE;
   use IEEE.std_logic_1164.all;
   use ieee.numeric_bit.all;
   use ieee.std_logic_arith.all;
library unisim ;
   use unisim.vcomponents.all ;

entity mb_hv_ctrl_top is 
port(    

      CPLD_MCU_AFE_RST   : in  std_logic;  -- connected to MCU port PA10, register reset
      CPLD_MCU_CAL_SS    : in  std_logic;  -- connected to MCU port  PE4, low active frame signal (like nCS)
      CPLD_MCU_AFE_SEL   : in  std_logic;  -- connected to MCU port  PA9, to distinguish between AFE and mDAB access
      CPLD_MCU_CAL_SCLK  : in  std_logic;  -- connected to MCU port  PE2, SPI clock
      CPLD_MCU_CAL_MOSI  : in  std_logic;  -- connected to MCU port  PE6, SPI data
		
      CPLD_MCU_USART6_RX : out std_logic;  -- connected to MCU port PG9, might also be used as CPLD_MCU_AFE_MISO(?)
      CPLD_MCU_USART6_TX : in  std_logic;  -- connected to MCU port PG14
                                       
      HV_UART_RX         : in  std_logic_vector(23 downto 0);   -- received from PMT base UART
		HV_UART_TX         : out std_logic_vector(23 downto 0);   -- to be sent to PMT base UART 
		
		AFE_EN             : out std_logic_vector(23 downto 0)    -- analog frontend power on / off 
		
     );
end mb_hv_ctrl_top ;

architecture arch_mb_hv_ctrl_top of mb_hv_ctrl_top is
 
 -- SPI Bus usage with MSB first:
 
 -- bit_31..29	reserved
 -- bit_28..24 AFE_UART_SEL_4..0
 -- bit_23..0	AFE_EN_23..0

 
 -- AFE_EN_xx outputs and UART mux update after falling edge of the 30th clock CPLD_MCU_CAL_SCLK
  
  signal spi_reg                : std_logic_vector (28 downto 0);
  signal afe_en_reg             : std_logic_vector (23 downto 0);
  signal uart_mux_reg           : std_logic_vector ( 4 downto 0);
  signal bit_ct                 : integer range 0 to 63;
  signal cpld_mcu_cal_sclk_bufg : std_logic;
  --signal cpld_mcu_afe_sel_bufg  : std_logic;
   
 begin 
 
  cpld_mcu_cal_sclk_bufg_inst : BUFG
    port map ( I => CPLD_MCU_CAL_SCLK, O => cpld_mcu_cal_sclk_bufg );

--  cpld_mcu_afe_sel_bufg_inst : BUFG
--    port map ( I => CPLD_MCU_AFE_SEL, O => cpld_mcu_afe_sel_bufg );
	
  
  GET_SPI_DATA: process(cpld_mcu_cal_sclk_bufg, CPLD_MCU_AFE_RST, CPLD_MCU_AFE_SEL, CPLD_MCU_CAL_SS)
   begin
	 if CPLD_MCU_AFE_RST = '1' then
	   bit_ct       <= 0;
		afe_en_reg   <= (others=>'0');
	   uart_mux_reg <= (others=>'0');
		spi_reg      <= (others=>'0');
	 else
	  if CPLD_MCU_AFE_SEL = '1' then
	   if CPLD_MCU_CAL_SS = '1' then -- low active !
	    bit_ct       <= 0; 
	   elsif  rising_edge(cpld_mcu_cal_sclk_bufg) then
		 if bit_ct < 29 then
		  bit_ct               <= bit_ct + 1;
		  spi_reg(0)           <= CPLD_MCU_CAL_MOSI;
		  spi_reg(28 downto 1) <= spi_reg(27 downto 0);
		 else
        bit_ct               <= 0;
        afe_en_reg           <= spi_reg(23 downto  0);
		  uart_mux_reg         <= spi_reg(28 downto 24); 		  
		 end if; 
		end if; --CPLD_MCU_CAL_SS = '1'
	  else bit_ct       <= 0; 
	  end if; --CPLD_MCU_AFE_SEL = '1'
    end if; --CPLD_MCU_AFE_RST = '1'
   end process GET_SPI_DATA;
	
	AFE_EN <= afe_en_reg;
	--AFE_EN <= X"ff_ffff";--afe_en_reg;
	--AFE_EN <= X"00_0000";--afe_en_reg;
	

   UART_MUX: process (uart_mux_reg, CPLD_MCU_USART6_TX, HV_UART_RX)
    begin
      case uart_mux_reg is 
         when "00000" => HV_UART_TX <= ( 0 => CPLD_MCU_USART6_TX, others=>'1'); CPLD_MCU_USART6_RX <= HV_UART_RX(0) ;
         when "00001" => HV_UART_TX <= ( 1 => CPLD_MCU_USART6_TX, others=>'1'); CPLD_MCU_USART6_RX <= HV_UART_RX(1) ;
         when "00010" => HV_UART_TX <= ( 2 => CPLD_MCU_USART6_TX, others=>'1'); CPLD_MCU_USART6_RX <= HV_UART_RX(2) ;
         when "00011" => HV_UART_TX <= ( 3 => CPLD_MCU_USART6_TX, others=>'1'); CPLD_MCU_USART6_RX <= HV_UART_RX(3) ;
         when "00100" => HV_UART_TX <= ( 4 => CPLD_MCU_USART6_TX, others=>'1'); CPLD_MCU_USART6_RX <= HV_UART_RX(4) ;
         when "00101" => HV_UART_TX <= ( 5 => CPLD_MCU_USART6_TX, others=>'1'); CPLD_MCU_USART6_RX <= HV_UART_RX(5) ;
         when "00110" => HV_UART_TX <= ( 6 => CPLD_MCU_USART6_TX, others=>'1'); CPLD_MCU_USART6_RX <= HV_UART_RX(6) ;
         when "00111" => HV_UART_TX <= ( 7 => CPLD_MCU_USART6_TX, others=>'1'); CPLD_MCU_USART6_RX <= HV_UART_RX(7) ;			
         when "01000" => HV_UART_TX <= ( 8 => CPLD_MCU_USART6_TX, others=>'1'); CPLD_MCU_USART6_RX <= HV_UART_RX(8) ;
         when "01001" => HV_UART_TX <= ( 9 => CPLD_MCU_USART6_TX, others=>'1'); CPLD_MCU_USART6_RX <= HV_UART_RX(9) ;
         when "01010" => HV_UART_TX <= (10 => CPLD_MCU_USART6_TX, others=>'1'); CPLD_MCU_USART6_RX <= HV_UART_RX(10);
         when "01011" => HV_UART_TX <= (11 => CPLD_MCU_USART6_TX, others=>'1'); CPLD_MCU_USART6_RX <= HV_UART_RX(11);
         when "01100" => HV_UART_TX <= (12 => CPLD_MCU_USART6_TX, others=>'1'); CPLD_MCU_USART6_RX <= HV_UART_RX(12);
         when "01101" => HV_UART_TX <= (13 => CPLD_MCU_USART6_TX, others=>'1'); CPLD_MCU_USART6_RX <= HV_UART_RX(13);
         when "01110" => HV_UART_TX <= (14 => CPLD_MCU_USART6_TX, others=>'1'); CPLD_MCU_USART6_RX <= HV_UART_RX(14);
         when "01111" => HV_UART_TX <= (15 => CPLD_MCU_USART6_TX, others=>'1'); CPLD_MCU_USART6_RX <= HV_UART_RX(15);				
         when "10000" => HV_UART_TX <= (16 => CPLD_MCU_USART6_TX, others=>'1'); CPLD_MCU_USART6_RX <= HV_UART_RX(16);
         when "10001" => HV_UART_TX <= (17 => CPLD_MCU_USART6_TX, others=>'1'); CPLD_MCU_USART6_RX <= HV_UART_RX(17);
         when "10010" => HV_UART_TX <= (18 => CPLD_MCU_USART6_TX, others=>'1'); CPLD_MCU_USART6_RX <= HV_UART_RX(18);
         when "10011" => HV_UART_TX <= (19 => CPLD_MCU_USART6_TX, others=>'1'); CPLD_MCU_USART6_RX <= HV_UART_RX(19);
         when "10100" => HV_UART_TX <= (20 => CPLD_MCU_USART6_TX, others=>'1'); CPLD_MCU_USART6_RX <= HV_UART_RX(20);
         when "10101" => HV_UART_TX <= (21 => CPLD_MCU_USART6_TX, others=>'1'); CPLD_MCU_USART6_RX <= HV_UART_RX(21);
         when "10110" => HV_UART_TX <= (22 => CPLD_MCU_USART6_TX, others=>'1'); CPLD_MCU_USART6_RX <= HV_UART_RX(22);
         when "10111" => HV_UART_TX <= (23 => CPLD_MCU_USART6_TX, others=>'1'); CPLD_MCU_USART6_RX <= HV_UART_RX(23);	
			
         when others  => HV_UART_TX     <= (others=>'1');      CPLD_MCU_USART6_RX <= '1';
			
      end case;
    end process UART_MUX;	
		       
   end arch_mb_hv_ctrl_top ;
