-------------------------------------------------------
-- Design Name      : mb_hv_mcu_signals_top 
-- File Name        : mb_hv_mcu_signals_top.vhd
-- Function         : used for simulation, generates MCU like signals
--                    
-- Coder            : K.-H. Sulanke, DESY, 2020-11-20
-------------------------------------------------------
-- Rev. 1


library IEEE;
   use IEEE.std_logic_1164.all;
   use ieee.numeric_bit.all;
   use ieee.std_logic_arith.all;
library unisim ;
   use unisim.vcomponents.all ;

entity mb_hv_mcu_signals_top is 
port(    
      clk                : in  std_logic;  --
		mcu_data           : in  std_logic_vector (28 downto 0); -- data to be sent
		done               : out std_logic;  -- SPI cycle done
      CPLD_MCU_CAL_SS    : out std_logic;  -- connected to MCU port  PE4, low active frame signal (like nCS)
      CPLD_MCU_AFE_SEL   : in  std_logic;  -- connected to MCU port  PA9, to distinguish between AFE and mDAB access
      CPLD_MCU_CAL_SCLK  : out std_logic;  -- connected to MCU port  PE2, SPI clock
      CPLD_MCU_CAL_MOSI  : out std_logic;  -- connected to MCU port  PE6, SPI data
      CPLD_MCU_USART6_TX : out std_logic;  -- connected to MCU port PG14
		HV_UART_RX         : out std_logic_vector(23 downto 0)   --
     );
end mb_hv_mcu_signals_top ;

architecture arch_mb_hv_mcu_signals_top of mb_hv_mcu_signals_top is
 
 -- SPI Bus usage with MSB first:
 
 -- bit_31..29	reserved
 -- bit_28..24 AFE_UART_SEL_4..0
 -- bit_23..0	AFE_EN_23..0

 -- AFE_EN_xx outputs and UART mux update after falling edge of the 30th clock CPLD_MCU_CAL_SCLK
  
   signal mcu_data_reg       : std_logic_vector (28 downto 0); -- shift register
   signal ct                 : integer range 0 to 255 := 0; 
   signal mcu_uart_tx        : std_logic_vector (9 downto 0); -- 
   signal tx_bit_ct          : integer range 0 to 15 := 0; 
	signal rx_bit_ct          : integer range 0 to 15 := 0; 
	signal CPLD_MCU_CAL_SS_nd : std_logic := '1';
	
	type   hv_uart_rxd_type is array (23 downto 0) of std_logic_vector (9 downto 0);
	signal hv_uart_rxd        : hv_uart_rxd_type;
	 
 begin 
 
 GEN_MCU_OUTPUT: process(clk, CPLD_MCU_AFE_SEL)
   begin
	 if CPLD_MCU_AFE_SEL = '0' then
	   CPLD_MCU_CAL_SS_nd <= '1';
		CPLD_MCU_CAL_SCLK <= '0';
	   ct       <= 0;
		done     <= '0';
		mcu_data_reg <= (others =>'0');
	 else
	  if rising_edge (clk) then	
	   if ct < 121 then
		 ct <= ct + 1;
	    if ct = 1 then
		  mcu_data_reg       <= mcu_data;
		  CPLD_MCU_CAL_SS_nd <= '0';
		  done               <= '0';
		 end if;
		 if    (ct mod 4 = 3) then
		   CPLD_MCU_CAL_SCLK <= '1';
		 elsif (ct mod 4 = 1) then
		   CPLD_MCU_CAL_SCLK <= '0';
		   if (ct /= 1) then
			 mcu_data_reg(28 downto 1) <= mcu_data_reg(27 downto 0);
			end if;
       end if; --(ct mod 4 = 3)
     else
       CPLD_MCU_CAL_SS_nd <= '1';
		 ct   <= 0;
		 done <= '1';
	  end if; --if ct < 121
	 end if; --rising_edge (clk) 
	end if; --CPLD_MCU_AFE_SEL = '0'
  end process GEN_MCU_OUTPUT;
 
  CPLD_MCU_CAL_MOSI <= mcu_data_reg(28);
  CPLD_MCU_CAL_SS   <= CPLD_MCU_CAL_SS_nd;

 GEN_MCU_UART_TX: process(clk, CPLD_MCU_CAL_SS_nd)
   begin
	 if CPLD_MCU_CAL_SS_nd = '0' then
	   tx_bit_ct <= 0;
		mcu_uart_tx <= (9 => '1', others =>'0');
	 else
	  if rising_edge (clk) then	  
	   if tx_bit_ct < 10 then
		 tx_bit_ct <= tx_bit_ct + 1;
		else
       tx_bit_ct <= 0;
      end if;	
      if tx_bit_ct = 0 then
		 mcu_uart_tx <= B"0111" & mcu_data(28 downto 24) & B"1" ;
		else
       mcu_uart_tx(9 downto 1) <= mcu_uart_tx(8 downto 0); 
      end if;		 
 	  end if; --rising_edge (clk) 
	 end if; --CPLD_MCU_CAL_SS_nd = '0'
   end process GEN_MCU_UART_TX;
 
   CPLD_MCU_USART6_TX <= mcu_uart_tx(9); 
	
	
	GEN_HV_UART_RX: process(clk, CPLD_MCU_CAL_SS_nd)
   begin
	 if CPLD_MCU_CAL_SS_nd = '0' then
	   rx_bit_ct <= 0;
		hv_uart_rxd <= (others => B"11_1111_1111");
	 else
	  if rising_edge (clk) then	  
	   if rx_bit_ct < 11 then
		 rx_bit_ct <= rx_bit_ct + 1;
		else
       rx_bit_ct <= 0;
      end if;
		if rx_bit_ct = 0 then
--         if    mcu_data(28 downto 24)= "00000" then hv_uart_rxd( 0)(9 downto 0) <= (B"0000" & mcu_data(28 downto 24) & B"1") ; 
--         elsif mcu_data(28 downto 24)= "00001" then hv_uart_rxd( 1)(9 downto 0) <= (B"0001" & mcu_data(28 downto 24) & B"1") ; 
--         elsif mcu_data(28 downto 24)= "00010" then hv_uart_rxd( 2)(9 downto 0) <= (B"0010" & mcu_data(28 downto 24) & B"1") ; 
--         elsif mcu_data(28 downto 24)= "00011" then hv_uart_rxd( 3)(9 downto 0) <= (B"0011" & mcu_data(28 downto 24) & B"1") ; 
--         elsif mcu_data(28 downto 24)= "00100" then hv_uart_rxd( 4)(9 downto 0) <= (B"0100" & mcu_data(28 downto 24) & B"1") ; 
--         elsif mcu_data(28 downto 24)= "00101" then hv_uart_rxd( 5)(9 downto 0) <= (B"0101" & mcu_data(28 downto 24) & B"1") ; 
--         elsif mcu_data(28 downto 24)= "00110" then hv_uart_rxd( 6)(9 downto 0) <= (B"0110" & mcu_data(28 downto 24) & B"1") ; 
--         elsif mcu_data(28 downto 24)= "00111" then hv_uart_rxd( 7)(9 downto 0) <= (B"0111" & mcu_data(28 downto 24) & B"1") ; 			
--         elsif mcu_data(28 downto 24)= "01000" then hv_uart_rxd( 8)(9 downto 0) <= (B"1000" & mcu_data(28 downto 24) & B"1") ; 
--         elsif mcu_data(28 downto 24)= "01001" then hv_uart_rxd( 9)(9 downto 0) <= (B"1001" & mcu_data(28 downto 24) & B"1") ; 
--         elsif mcu_data(28 downto 24)= "01010" then hv_uart_rxd(10)(9 downto 0) <= (B"1010" & mcu_data(28 downto 24) & B"1") ; 
--         elsif mcu_data(28 downto 24)= "01011" then hv_uart_rxd(11)(9 downto 0) <= (B"1011" & mcu_data(28 downto 24) & B"1") ; 
--         elsif mcu_data(28 downto 24)= "01100" then hv_uart_rxd(12)(9 downto 0) <= (B"1100" & mcu_data(28 downto 24) & B"1") ; 
--         elsif mcu_data(28 downto 24)= "01101" then hv_uart_rxd(13)(9 downto 0) <= (B"1101" & mcu_data(28 downto 24) & B"1") ; 
--         elsif mcu_data(28 downto 24)= "01110" then hv_uart_rxd(14)(9 downto 0) <= (B"1110" & mcu_data(28 downto 24) & B"1") ; 
--         elsif mcu_data(28 downto 24)= "01111" then hv_uart_rxd(15)(9 downto 0) <= (B"1111" & mcu_data(28 downto 24) & B"1") ; 				
--         elsif mcu_data(28 downto 24)= "10000" then hv_uart_rxd(16)(9 downto 0) <= (B"0000" & mcu_data(28 downto 24) & B"1") ; 
--         elsif mcu_data(28 downto 24)= "10001" then hv_uart_rxd(17)(9 downto 0) <= (B"0001" & mcu_data(28 downto 24) & B"1") ; 
--         elsif mcu_data(28 downto 24)= "10010" then hv_uart_rxd(18)(9 downto 0) <= (B"0010" & mcu_data(28 downto 24) & B"1") ; 
--         elsif mcu_data(28 downto 24)= "10011" then hv_uart_rxd(19)(9 downto 0) <= (B"0011" & mcu_data(28 downto 24) & B"1") ; 
--         elsif mcu_data(28 downto 24)= "10100" then hv_uart_rxd(20)(9 downto 0) <= (B"0100" & mcu_data(28 downto 24) & B"1") ; 
--         elsif mcu_data(28 downto 24)= "10101" then hv_uart_rxd(21)(9 downto 0) <= (B"0101" & mcu_data(28 downto 24) & B"1") ; 
--         elsif mcu_data(28 downto 24)= "10110" then hv_uart_rxd(22)(9 downto 0) <= (B"0110" & mcu_data(28 downto 24) & B"1") ; 
--         elsif mcu_data(28 downto 24)= "10111" then hv_uart_rxd(23)(9 downto 0) <= (B"0111" & mcu_data(28 downto 24) & B"1") ; 	
--		   end if;
         --else                                       hv_uart_rxd(23 downto 0)    <= (others => B"11_1111_1111");
		 
        case mcu_data(28 downto 24) is 
         when "00000" => hv_uart_rxd( 0)(9 downto 0) <= (B"0000" & mcu_data(28 downto 24) & B"1") ; 
         when "00001" => hv_uart_rxd( 1)(9 downto 0) <= (B"0001" & mcu_data(28 downto 24) & B"1") ; 
         when "00010" => hv_uart_rxd( 2)(9 downto 0) <= (B"0010" & mcu_data(28 downto 24) & B"1") ; 
         when "00011" => hv_uart_rxd( 3)(9 downto 0) <= (B"0011" & mcu_data(28 downto 24) & B"1") ; 
         when "00100" => hv_uart_rxd( 4)(9 downto 0) <= (B"0100" & mcu_data(28 downto 24) & B"1") ; 
         when "00101" => hv_uart_rxd( 5)(9 downto 0) <= (B"0101" & mcu_data(28 downto 24) & B"1") ; 
         when "00110" => hv_uart_rxd( 6)(9 downto 0) <= (B"0110" & mcu_data(28 downto 24) & B"1") ; 
         when "00111" => hv_uart_rxd( 7)(9 downto 0) <= (B"0111" & mcu_data(28 downto 24) & B"1") ; 			
         when "01000" => hv_uart_rxd( 8)(9 downto 0) <= (B"1000" & mcu_data(28 downto 24) & B"1") ; 
         when "01001" => hv_uart_rxd( 9)(9 downto 0) <= (B"1001" & mcu_data(28 downto 24) & B"1") ; 
         when "01010" => hv_uart_rxd(10)(9 downto 0) <= (B"1010" & mcu_data(28 downto 24) & B"1") ; 
         when "01011" => hv_uart_rxd(11)(9 downto 0) <= (B"1011" & mcu_data(28 downto 24) & B"1") ; 
         when "01100" => hv_uart_rxd(12)(9 downto 0) <= (B"1100" & mcu_data(28 downto 24) & B"1") ; 
         when "01101" => hv_uart_rxd(13)(9 downto 0) <= (B"1101" & mcu_data(28 downto 24) & B"1") ; 
         when "01110" => hv_uart_rxd(14)(9 downto 0) <= (B"1110" & mcu_data(28 downto 24) & B"1") ; 
         when "01111" => hv_uart_rxd(15)(9 downto 0) <= (B"1111" & mcu_data(28 downto 24) & B"1") ; 				
         when "10000" => hv_uart_rxd(16)(9 downto 0) <= (B"0000" & mcu_data(28 downto 24) & B"1") ; 
         when "10001" => hv_uart_rxd(17)(9 downto 0) <= (B"0001" & mcu_data(28 downto 24) & B"1") ; 
         when "10010" => hv_uart_rxd(18)(9 downto 0) <= (B"0010" & mcu_data(28 downto 24) & B"1") ; 
         when "10011" => hv_uart_rxd(19)(9 downto 0) <= (B"0011" & mcu_data(28 downto 24) & B"1") ; 
         when "10100" => hv_uart_rxd(20)(9 downto 0) <= (B"0100" & mcu_data(28 downto 24) & B"1") ; 
         when "10101" => hv_uart_rxd(21)(9 downto 0) <= (B"0101" & mcu_data(28 downto 24) & B"1") ; 
         when "10110" => hv_uart_rxd(22)(9 downto 0) <= (B"0110" & mcu_data(28 downto 24) & B"1") ; 
         when "10111" => hv_uart_rxd(23)(9 downto 0) <= (B"0111" & mcu_data(28 downto 24) & B"1") ; 	
			
         when others  => hv_uart_rxd(23 downto 0)     <= (others => B"11_1111_1111");
        end case;
		else
       hv_uart_rxd( 0)(9 downto 1) <= hv_uart_rxd( 0)(8 downto 0); 
		 hv_uart_rxd( 1)(9 downto 1) <= hv_uart_rxd( 1)(8 downto 0); 
		 hv_uart_rxd( 2)(9 downto 1) <= hv_uart_rxd( 2)(8 downto 0); 
		 hv_uart_rxd( 3)(9 downto 1) <= hv_uart_rxd( 3)(8 downto 0); 
		 hv_uart_rxd( 4)(9 downto 1) <= hv_uart_rxd( 4)(8 downto 0); 
		 hv_uart_rxd( 5)(9 downto 1) <= hv_uart_rxd( 5)(8 downto 0); 
		 hv_uart_rxd( 6)(9 downto 1) <= hv_uart_rxd( 6)(8 downto 0); 
		 hv_uart_rxd( 7)(9 downto 1) <= hv_uart_rxd( 7)(8 downto 0);   	 
	    hv_uart_rxd( 8)(9 downto 1) <= hv_uart_rxd( 8)(8 downto 0); 
		 hv_uart_rxd( 9)(9 downto 1) <= hv_uart_rxd( 9)(8 downto 0); 
		 hv_uart_rxd(10)(9 downto 1) <= hv_uart_rxd(10)(8 downto 0); 
		 hv_uart_rxd(11)(9 downto 1) <= hv_uart_rxd(11)(8 downto 0); 
		 hv_uart_rxd(12)(9 downto 1) <= hv_uart_rxd(12)(8 downto 0); 
		 hv_uart_rxd(13)(9 downto 1) <= hv_uart_rxd(13)(8 downto 0); 
		 hv_uart_rxd(14)(9 downto 1) <= hv_uart_rxd(14)(8 downto 0); 
		 hv_uart_rxd(15)(9 downto 1) <= hv_uart_rxd(15)(8 downto 0); 
		 hv_uart_rxd(16)(9 downto 1) <= hv_uart_rxd(16)(8 downto 0); 
		 hv_uart_rxd(17)(9 downto 1) <= hv_uart_rxd(17)(8 downto 0); 
		 hv_uart_rxd(18)(9 downto 1) <= hv_uart_rxd(18)(8 downto 0); 
		 hv_uart_rxd(19)(9 downto 1) <= hv_uart_rxd(19)(8 downto 0); 
		 hv_uart_rxd(20)(9 downto 1) <= hv_uart_rxd(20)(8 downto 0); 
		 hv_uart_rxd(21)(9 downto 1) <= hv_uart_rxd(21)(8 downto 0); 
		 hv_uart_rxd(22)(9 downto 1) <= hv_uart_rxd(22)(8 downto 0); 
		 hv_uart_rxd(23)(9 downto 1) <= hv_uart_rxd(23)(8 downto 0); 
      end if;	--rx_bit_ct = 0	 
 	  end if; --rising_edge (clk) 
	 end if; --CPLD_MCU_CAL_SS_nd = '0'
   end process GEN_HV_UART_RX;
 
--   HV_UART_RX(23 downto 0) <= hv_uart_rxd(23 downto 0)(9); 
	HV_UART_RX( 0) <= hv_uart_rxd( 0)(9); --
	HV_UART_RX( 1) <= hv_uart_rxd( 1)(9); --
	HV_UART_RX( 2) <= hv_uart_rxd( 2)(9); --
	HV_UART_RX( 3) <= hv_uart_rxd( 3)(9); --
	HV_UART_RX( 4) <= hv_uart_rxd( 4)(9); --
	HV_UART_RX( 5) <= hv_uart_rxd( 5)(9); --
	HV_UART_RX( 6) <= hv_uart_rxd( 6)(9); --
	HV_UART_RX( 7) <= hv_uart_rxd( 7)(9); --
	HV_UART_RX( 8) <= hv_uart_rxd( 8)(9); --
	HV_UART_RX( 9) <= hv_uart_rxd( 9)(9); --
	HV_UART_RX(10) <= hv_uart_rxd(10)(9); --
	HV_UART_RX(11) <= hv_uart_rxd(11)(9); --
	HV_UART_RX(12) <= hv_uart_rxd(12)(9); --
	HV_UART_RX(13) <= hv_uart_rxd(13)(9); --
	HV_UART_RX(14) <= hv_uart_rxd(14)(9); --
	HV_UART_RX(15) <= hv_uart_rxd(15)(9); --
	HV_UART_RX(16) <= hv_uart_rxd(16)(9); --
	HV_UART_RX(17) <= hv_uart_rxd(17)(9); --
	HV_UART_RX(18) <= hv_uart_rxd(18)(9); --
	HV_UART_RX(19) <= hv_uart_rxd(19)(9); --
	HV_UART_RX(20) <= hv_uart_rxd(20)(9); --
	HV_UART_RX(21) <= hv_uart_rxd(21)(9); --
	HV_UART_RX(22) <= hv_uart_rxd(22)(9); --
	HV_UART_RX(23) <= hv_uart_rxd(23)(9); --

   end arch_mb_hv_mcu_signals_top ;
